package com.rf.youth.user.rest;

import com.alibaba.fastjson.JSONObject;
import com.rf.youth.base.rest.BaseController;
import com.rf.youth.security.AesEncryptUtils;
import com.rf.youth.security.SafetyProcess;
import com.rf.youth.user.dao.model.UserEntity;
import com.rf.youth.user.service.SystemService;
import com.rf.youth.user.service.UserService;
import com.rf.youth.utils.JWTUtil;
import com.rf.youth.utils.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * @Description: 后台首页、登录、版本等接口
 * @Author: zsf
 * @Date: 2022/7/4
 */
@RestController
@RequestMapping("/system")
@Api(tags = "后台首页、登录、版本等接口")
public class SystemController extends BaseController {

    @Autowired
    private SystemService systemService;

    @Autowired
    private UserService userService;




    @Value("${spring.profiles.active}")
    private String profileValue;


    @GetMapping("/getVersion")
    @SafetyProcess
    public Result getSystemVersion(){
        return success(systemService.getSystemVersion(),profileValue);
    }


//    /**
//     * 注册用户
//     * @param jsonParam
//     * @return
//     */
//    @PostMapping("/registerUser")
//    @ApiOperation(value = "用户注册",notes = "json字符串形式传参（加密）,data参数包括:data:注册用户基本信息《birthday：生日，gender:性别，password:密码，roleType:角色，institutionNo:机构编号，phone:账号或电话》，authCode:验证码（公网版必填），" +
//            "verification:验证信息《beginTime:时间，authCode:验证码，phone:电话》")
//    @SafetyProcess
//    public Result registerUser(@RequestBody String jsonParam){
//        try {
//            String data = AesEncryptUtils.decrypt(JSONObject.parseObject(jsonParam).getString("data"));
//            JSONObject jsonData = JSONObject.parseObject(data);
//            JSONObject jsonUserEntity =jsonData.getJSONObject("data");
//            UserEntity userEntity = jsonUserEntity.toJavaObject(UserEntity.class);
//            userEntity.setModelPhone(Constant.DEFAULT_VALUE_ZERO);
//            String institutionNo = userEntity.getInstitutionNo();
//            if (StringUtils.isEmpty(institutionNo)){
//                return fail("","机构编号不能为空");
//            }
//            //判断机构编号是否存在
//            InstitutionEntity institutionEntity = this.institutionService.findByInstitutionNo(institutionNo);
//            if (institutionEntity == null) {
//                return fail("", "机构编号不存在！");
//            }
//            userEntity.setInstitutionName(institutionEntity.getInstitutionName());
//            if (UserRole.COMMON.getType().equals(userEntity.getRoleType())){
//
//            }
//            //区分是否需要验证码
//            if (Constant.WEB_INSTITUTION_CODE.equals(userEntity.getInstitutionNo())){
//                String authCode = jsonData.getString("authCode");
//                JSONObject jsonVerification = jsonData.getJSONObject("verification");
//                String jsonPhone = jsonVerification.getString("phone");
//                String jsonAuthCode = jsonVerification.getString("authCode");
//                String jsonBeginTime = jsonVerification.getString("beginTime");
//                if (StringUtils.isEmpty(jsonBeginTime)  || StringUtils.isEmpty(jsonAuthCode) ) {
//                    return fail("", "请重新获取验证码");
//                }
//                //计算时间差---分钟
//                //int diff = DateUtil.getDistanceByUnit(DateUtil.parseDate(jsonBeginTime,DateUtil.ACCURACY_PATTERN_MINUTE), DateUtil.getCurrentDate(), 2);
//                long diff = DateUtil.between(DateUtil.date(), DateUtil.parse(jsonBeginTime,PURE_DATETIME_PATTERN), DateUnit.MINUTE);
//                if (diff < 0 || diff > 5) {
//                    return fail("", "验证码已失效");
//                }
//                if (!jsonAuthCode.equals(authCode) || !jsonPhone.equals(userEntity.getPhone()) ) {
//                    return fail("", "验证码错误");
//                }
//                userEntity.setUserStatus(UserStatus.PASS.getType());
//            }
//            UserEntity userInfo = this.userService.findPhoneAndInstitutionNoAndRoleType(userEntity.getPhone(), userEntity.getInstitutionNo(),userEntity.getRoleType());
//            if (userInfo == null) {
//                userEntity.setPassword(DigestUtils.md5DigestAsHex(userEntity.getPassword().getBytes()));
//                this.userService.save(userEntity);
//                return success();
//            } else {
//                return fail("", "账号已注册");
//            }
//        }catch (Exception e){
//            e.printStackTrace();
//            return fail();
//        }
//    }
    @SafetyProcess
    @ApiOperation(value = "系统登录接口",notes = "data参数包括：phone:账号, password:密码,institutionNo:机构编号,roleType:用户角色")
    @PostMapping("/login")
    public Result login(HttpServletRequest request, @RequestBody String jsonParams){
        try {
            JSONObject jsonObject =JSONObject.parseObject( AesEncryptUtils.decrypt(JSONObject.parseObject(jsonParams).getString("data")));
            if (!jsonObject.containsKey("phone") || !jsonObject.containsKey("password") || !jsonObject.containsKey("institutionNo")|| !jsonObject.containsKey("roleType")) {
                return fail("", "账号或密码或所属机构不能为空");
            }
            String password = jsonObject.getString("password");
            String username = jsonObject.getString("username");
            if (StringUtils.isEmpty(password) || StringUtils.isEmpty( username )) {
                return fail("", "账号或密码不能为空");
            }
            UserEntity userEntity = this.userService.findUserByName(username);
            if (userEntity == null) {
                return fail(null, "用户不存在");
            }
            if (userEntity.getPassword().equals(password)) {
                //生成token
                userEntity.setPassword(null);
                String token = JWTUtil.getTokenByUserInfo(userEntity);
                // 创建返回的json对象
                JSONObject resultJson = new JSONObject();
                resultJson.put("user", userEntity);
                resultJson.put("token", token);
                request.getSession().setAttribute("user", userEntity);
                return success(resultJson);
            } else {
                return fail(null, "密码错误");
            }
        }catch (Exception e){
            e.printStackTrace();
            return fail();
        }
    }


//    /**
//     * 忘记密码
//     * @param jsonParam 参数
//     * @desc 局域网和公网版 区别：公网版需要验证码参数
//     * @return
//     */
//    @SafetyProcess
//    @ApiOperation(value = "忘记密码",notes = "json字符串形式传参（加密）,data参数包括:password:新密码,institutionNo:机构编号,phone:账号或电话,roleType:用户角色,authCode:验证码（公网版必填），" +
//            "verification:验证信息《beginTime:时间,authCode:验证码,phone:电话》")
//    @PostMapping("/forgotPassword")
//    public Result forgotPassword(@RequestBody String jsonParam){
//        try {
//            JSONObject dataParam = JSONObject.parseObject(AesEncryptUtils.decrypt(JSONObject.parseObject(jsonParam).getString("data")));
//            String phone = dataParam.getString("phone");
//            String password = dataParam.getString("password");
//            String institutionNo = dataParam.getString("institutionNo");
//            String roleType = dataParam.getString("roleType");
//            if ( StringUtils.isEmpty( phone)  || StringUtils.isEmpty( password )  || StringUtils.isEmpty( institutionNo ) ) {
//                return fail("", "请按要求填写所需信息");
//            }
//            UserEntity userEntity = this.userService.findPhoneAndInstitutionNoAndRoleType(phone,institutionNo,roleType);
//            if (userEntity == null) {
//                return fail("", "请先注册");
//            }
//            userEntity.setPassword(password);
//            //区分公网版和非公网版
//            if (Constant.WEB_INSTITUTION_CODE.equals(institutionNo)){
//                String authCode = dataParam.getString("authCode");
//                JSONObject jsonVerification = dataParam.getJSONObject("verification");
//                String jsonPhone = jsonVerification.getString("phone");
//                String jsonAuthCode = jsonVerification.getString("authCode");
//                String jsonBeginTime = jsonVerification.getString("beginTime");
//                if (StringUtils.isEmpty(jsonBeginTime ) || StringUtils.isEmpty(jsonAuthCode) ) {
//                    return fail("", "请重新获取验证码");
//                }
//                //计算时间差---分钟
//                //int diff = DateUtil.getDistanceByUnit(DateUtil.parseDate(jsonBeginTime), DateUtil.getCurrentDate(), 2);
//                long diff = DateUtil.between(DateUtil.date(), DateUtil.parse(jsonBeginTime,PURE_DATETIME_PATTERN), DateUnit.MINUTE);
//                if (diff < 0 || diff > 5) {
//                    return fail("", "验证码已失效");
//                }
//                if (jsonAuthCode.equals(authCode) && jsonPhone.equals(phone)) {
//                   // this.userService.updatePassword(phone, password, institutionNo,roleType);
//                    this.userService.save(userEntity);
//                } else {
//                    return fail("", "验证码错误");
//                }
//            }else {
//                this.userService.save(userEntity);
//               // this.userService.updatePassword(phone, password, institutionNo,roleType);
//            }
//            return success();
//        }catch (Exception e){
//            e.printStackTrace();
//            return fail();
//        }
//    }

    /**
     * 修改密码
     * @param jsonParam 待修改用户密码参数
     * @return
     */
    @SafetyProcess
    @ApiOperation(value = "修改密码",notes = "data参数包括：id:用户id,password:新密码,oldPassword:旧密码")
    @PostMapping("/updatePassword")
    public Result updatePassword(@RequestBody String jsonParam){
        try {
            JSONObject jsonObject = JSONObject.parseObject(AesEncryptUtils.decrypt(JSONObject.parseObject(jsonParam).getString("data")));
            if(!jsonObject.containsKey("id")|| StringUtils.isEmpty(jsonObject.getString("id")))
                return failBadRequest(null,"用户id不能为空！");
            if(!jsonObject.containsKey("password")|| StringUtils.isEmpty(jsonObject.getString("password")))
                return failBadRequest(null,"登录密码不能为空！");
            String password =  jsonObject.getString("password");
            String oldPassword = jsonObject.getString("oldPassword");
            String id = jsonObject.getString("id");
            UserEntity userEntity = userService.findUserById(id);
            if (!userEntity.getPassword().equals(oldPassword)) {
                return fail("", "旧密码有误");
            }
            userEntity.setPassword(password);
            this.userService.saveUser(userEntity);
            return success();
        }catch (Exception e){
            e.printStackTrace();
            return fail();
        }
    }
}
