package com.rf.youth.user.rest;

import com.rf.youth.base.rest.BaseController;
import com.rf.youth.opLog.annotation.OperationLogAnnotation;
import com.rf.youth.security.SafetyProcess;
import com.rf.youth.user.dao.model.UserEntity;
import com.rf.youth.user.service.UserService;
import com.rf.youth.utils.Constant;
import com.rf.youth.utils.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.DigestUtils;
import org.springframework.web.bind.annotation.*;
import java.nio.charset.StandardCharsets;


/**
 * @author zsy
 * @description:登录认证
 * @date 2021/6/17 15:55
 */
@Slf4j
@RestController
@RequestMapping("/user")
@Api(tags = "用户操作")
public class UserController extends BaseController {

    @Autowired
    private UserService userService;




    /**
     * 重置密码
     * @return
     */
    @SafetyProcess
    @PostMapping("/resetPassword/{userId}")
    @OperationLogAnnotation
    @ApiOperation("用户重置密码")
    public Result resetPassword( @PathVariable String userId) throws Exception {

        if (StringUtils.isEmpty(userId) ) {
            return fail("", "用户id不能为空");
        }
        UserEntity userEntity = this.userService.findUserById(userId);
        if (userEntity == null) {
            return fail("", "请先注册");
        } else {
            userEntity.setPassword(DigestUtils.md5DigestAsHex(Constant.DEFAULT_PASSWORD.getBytes(StandardCharsets.UTF_8)));
            userService.saveUser(userEntity);

            return success();
        }
    }







}
